﻿using System;
using ServiceReference;
using System.Reflection;
using System.Xml.Serialization;

namespace tutorial
{
    class Program
    {
        string securityID = "";
        DateTime tokenValidUntil = DateTime.Now;

        SihotServices01PortTypeClient client = new SihotServices01PortTypeClient(SihotServices01PortTypeClient.EndpointConfiguration.SihotServices01Port, Environment.GetEnvironmentVariable("WSURL") );

        static void Main(string[] args)
        {
            MethodInfo method = typeof(XmlSerializer).GetMethod("set_Mode", BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Static);
            method.Invoke(null, new object[] { 1 });

            Console.WriteLine("Starting up");

            var p = new Program();
            var until = p.getSecurityID();
            Console.WriteLine(until);

            Console.WriteLine(p.tokenValidUntil);
            Console.WriteLine("Searching inhouse guests for " + DateTime.Now.ToString("yyyy-MM-dd"));
            p.getGuestInHouseGuests(DateTime.Now.ToString("yyyy-MM-dd"));

            Console.WriteLine("Finish");
        }

        String getSecurityID()
        {
            if ( securityID.Length > 0 &&  tokenValidUntil < DateTime.Now.AddSeconds(20) )
            {
                return securityID;
            }

            AuthenticateRequest request = new AuthenticateRequest();
            request.AuthenticationInfos = new AuthenticateRequestAuthenticationInfos()
            {
                user = Environment.GetEnvironmentVariable("SIHOT_USER"),
                password = Environment.GetEnvironmentVariable("SIHOT_PASSWORD"),
                hotel = Environment.GetEnvironmentVariable("SIHOT_HOTEL")
            };

            try
            {
                var response = client.Authenticate(request);
                Console.Write("securityID received: ");
                Console.WriteLine(response.Authentication[0].SecurityID);

                securityID = response.Authentication[0].SecurityID;
                tokenValidUntil = DateTime.Now.AddSeconds(Int32.Parse(response.Authentication[0].DurationInSec));

                return securityID;
            }
            catch (Exception e)
            {
                Console.WriteLine(e);
                return "";
            }
        }

        S_GUEST_IN_HOUSE_SEARCH_V001Response getGuestInHouseGuests(String callingDate)
        {
            var req = new S_GUEST_IN_HOUSE_SEARCH_V001Request()
            {
                Authentication = new Authentication()
                {
                    SecurityID = securityID
                },
                GuestInHouseSearch = new S_GUEST_IN_HOUSE_SEARCH_V001RequestGuestInHouseSearch()
                {
                    date = callingDate
                }
            };

            var res = client.S_GUEST_IN_HOUSE_SEARCH_V001(req);

            if  (! res.Result.Success.Equals("true") )
            {
                Console.WriteLine(res.Result.Success);
                foreach( MSGENTRY msg in res.Result.MSGLIST )
                {
                    Console.Write(msg.MSGNO + " :");
                    Console.WriteLine(msg.MSGTEXT);
                } 
                Console.WriteLine("Something is not right");
                return res;
            }

            showInhouseGuests(res);
            return res;
        }

        void showInhouseGuests(S_GUEST_IN_HOUSE_SEARCH_V001Response res)
        {
            foreach ( S_GUEST_IN_HOUSE_SEARCH_V001ResponseRoom r in res.room )
            {
                Console.Write( "Room type: " + r.category +  ": ");
                foreach( S_GUEST_IN_HOUSE_SEARCH_V001ResponseRoomPerson p in r.Person )
                {
                    Console.WriteLine(p.name1 + ", " + p.name2);
                }
            }
        }
    }
}
